member(X,[X|_]).
member(X,[_|Y]) :- member(X,Y).

memberD(X,[X|_]) :- !.
memberD(X,[_|Y]) :- memberD(X,Y), !.

append([],X,X).
append([A|X],B,[A|Y]) :- append(X,B,Y).

appendD([],X,X) :- !.
appendD([A|X],B,[A|Y]) :- appendD(X,B,Y), !.

writelist([]) :- ttyflush.
writelist([nl|B]) :- !, nl, writelist(B).
writelist([A|B]) :- write(A), writelist(B).

writef(L) :- write(L), ttyflush, !.

subset([], _) :- !.
subset([A|B], Set) :- 
	member(A, Set),
	subset(B, Set),
	!.

const_to_list(Constant, List) :-
	name(Constant, Ascii),
	name2(List, Ascii),
	!.

name2([], []) :- !.
name2([A|As], [N|Ans]) :-
	name(A, [N]),
	name2(As, Ans),
	!.

list_to_const(Constant, List) :-
	name2(List, Ascii),
	name(Constant, Ascii),
	!.

remove(_, [], []) :- !.
remove(A, [A|B], B) :- !.
remove(A, [X|B], [X|C]) :- remove(A, B, C), !.



remove_all(_, [], []) :- !.
remove_all(A, [Z|B], C) :- \+ \+ A=Z,!, remove_all(A,B,C).
remove_all(A, [X|B], [X|C]) :- remove_all(A, B, C), !.

remove_list([], New, New).
remove_list([A|R], List, New) :-
	remove_all(A, List, List2),
	remove_list(R, List2, New).
	

size(L,N) :- size(L,0,N), !.
	
size([],N,N) :- !.
size([_|B],M,N) :- 
	P is M + 1,
	size(B,P,N),
	!.


revsort(List, Rev) :-
	sort(List, List2),
	reverse(List2, Rev),
	!.

reverse([],[]).
reverse([A|B], R) :- reverse(B,Z), append(Z,[A],R), !.

remove_dups([], []).
remove_dups([A|R], [A|S]) :- 
	!,
	remove_all(A, R, R2),
	remove_dups(R2, S).





partition([], _, [], []).
partition([[A|C]|R], A, [[A|C]|S], T) :-
	!,
	partition(R, A, S, T).
partition([[B|C]|R], A, S, [[B|C]|T]) :-
	partition(R, A, S, T).

get_dups([], Dups, Dups) :- !.
get_dups([A|Rest], Dups, Dups2) :-
	member(A, Rest),
	\+ member(A, Dups),
	get_dups(Rest, [A|Dups], Dups2),
	!.
get_dups([_|Rest], Dups, Dups2) :-
	get_dups(Rest, Dups, Dups2),
	!.

intersect([],_,[]).
intersect([X|R], Y, [X|Z]) :-
	member(X, Y),
	!,
	intersect(R, Y, Z).
intersect([_|R], Y, Z) :-
	intersect(R, Y, Z).

prepend_all(_, [], []).
prepend_all(A, [X|R], [Y|S]) :-
	appendD(A, X, Y),
	prepend_all(A, R, S).

append_all([], _, []).
append_all([X|R], A, [Y|S]) :-
	appendD(X, A, Y),
	append_all(R, A, S).

combs(P,N) :- perm_terms(P,L), size(L,N).

